pub fn split<'a>(regex: &Regex, some_string: &'a str) -> Vec<&'a str>Expand description
Splits a string by each match of given regular expression. An empty vector is returned if the regular expression does not match of is faulty.
Arguments:
regex- fancy_regex::Regex which matches the split positions.some_string- String to split.